# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json

import requests

from .base import BaseChannel
from channels.exceptions import HttpError, ImproperlyConfigured


class SlackChannel(BaseChannel):
    def __init__(self, config):
        # Required
        self.load_required_config(config, {
            "URL": "url"
        })

        # Optional
        self.load_optional_config(config, {
            "USERNAME": "username",
            "CHANNEL": "channel",
            "ICON_EMOJI": "icon_emoji",
            "ICON_URL": "icon_url"
        })

        # Validation
        if hasattr(self, "icon_emoji") and hasattr(self, "icon_url"):
            raise ImproperlyConfigured(
                "Must not set both ICON_EMOJI and ICON_URL")

    def send(self, message, fail_silently=False, options=None):
        payload = {
            'text': message
        }
        if hasattr(self, "channel"):
            payload["channel"] = self.channel
        if hasattr(self, "username"):
            payload["username"] = self.username
        if hasattr(self, "icon_url"):
            payload["icon_url"] = self.icon_url
        if hasattr(self, "icon_emoji"):
            payload["icon_emoji"] = self.icon_emoji
        data = {
            'payload': json.dumps(payload)
        }
        try:
            response = requests.post(self.url, data=data)
            if response.status_code != requests.codes.ok:
                raise HttpError(response.status_code)
        except:
            if not fail_silently:
                raise



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import requests

from .base import BaseChannel
from channels.exceptions import HttpError


class HipChatChannel(BaseChannel):
    def __init__(self, config):
        # Required
        self.load_required_config(config, {
            "API_ID": "api_id",
            "TOKEN": "token"
        })

        # Optional
        self.load_optional_config(config, {
            "BASE_URL": "base_url"
        })

        if not hasattr(self, "base_url"):
            self.base_url = "https://api.hipchat.com/v2/"
        self.url = "{0}room/{1}/notification?auth_token={2}".format(
            self.base_url, self.api_id, self.token)

    def send(self, message, fail_silently=False, options=None):
        data = {
            'message': message
        }
        try:
            response = requests.post(self.url, data=data)
            if response.status_code != requests.codes.no_content:
                raise HttpError(response.status_code)
        except:
            if not fail_silently:
                raise



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import sys

from django.conf import settings


CHANNELS = settings.CHANNELS["CHANNELS"]


def _load_module(name):
    __import__(name)
    return sys.modules[name]


def _load_backend(name):
    module_name, klass_name = name.rsplit(".", 1)
    module = _load_module(module_name)
    return getattr(module, klass_name)


def send(message, fail_silently=False):
    """Send a notification to all configured backends

    :param message: A message
    :type message: str | unicode
    :type fail_silently: bool
    """
    for klass, config in CHANNELS.items():
        channel = _load_backend(klass)(config)
        channel.send(message, fail_silently=fail_silently)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from abc import ABCMeta, abstractmethod

from six import add_metaclass

from channels.exceptions import ImproperlyConfigured


@add_metaclass(ABCMeta)
class BaseChannel(object):
    @abstractmethod
    def send(self, message, fail_silently=False, options=None):
        pass

    def load_required_config(self, config, key_attrs):
        for key, attr in key_attrs.items():
            value = config.get(key, None)
            if value is None:
                raise ImproperlyConfigured("Required configuration: " + key)
            setattr(self, attr, value)

    def load_optional_config(self, config, key_attrs):
        for key, attr in key_attrs.items():
            value = config.get(key, None)
            if value is not None:
                setattr(self, attr, value)



#!/usr/bin/env python
import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


__version__ = '0.1.0'

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

requires = [
    'Django>=1.7,<1.9',
    'requests>=2.7.0',
    'six>=1.9.0'
]

extras_require = {
    'docs': [
        'Sphinx<1.4,>=1.3',
        'sphinx-rtd-theme<0.2,>=0.1.8'
    ]
}

classifiers = [
    'Development Status :: 3 - Alpha',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'Operating System :: OS Independent',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Topic :: Internet :: WWW/HTTP',
]

setup(
    name='django-channels',
    version=__version__,
    packages=['channels', 'channels.backends'],
    include_package_data=True,
    license='MIT',
    description='A Django library for sending notifications',
    long_description=README,
    url='https://github.com/ymyzk/django-channels',
    author='Yusuke Miyazaki',
    author_email='miyazaki.dev@gmail.com',
    install_requires=requires,
    extras_require=extras_require,
    classifiers=classifiers
)
